/*
 * $URL$
 * 
 * Copyright (C) 2022 by Arizona State University and Mark Robinson.
 * All rights reserved.
 *
 * $Author$
 *   $Date$
 *    $Rev$
 *
 * (description of this file here)
 *
 */
#include "image_struct.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

struct shadowcam_image* init_shadowcam_image(const char* filename) {
  puts("Initializing ShadowCam image struct.");

  struct shadowcam_image* image = malloc(sizeof(struct shadowcam_image));
  if (image == NULL) {
    printf("malloc failed at %s:%d\n", __FILE__, __LINE__);
    exit(1);
  }

  image->filename = strdup(filename);
  image->file = fopen(filename, "rb");
  if (image->file == NULL) {
    printf("fopen of %s failed with errno %d\n", filename, errno);
    exit(1);
  }

  image->data_offset = 0;
  image->tdi_direction = '\0';
  for(int i=0; i < SHC_CHANNELS; i++) {
    image->gain[i] = 9999;
    image->offset[i] = 9999;
  }

  for(int i=0; i<6; i++) {
    image->bterm[i] = 9999;
    image->xterm[i] = 9999;
  }
  image->bterm[0] = 0;

  image->fpa_a_temp = 999999;
  image->line_rate_ms = 999999;
  image->lines = 999999;
  image->data = NULL;

  image->samples = 9999;
  image->target_name = NULL;
  image->line_rate_code = 9999;
  image->preroll = 9999;
  image->klines = 9999;
  image->preroll_time = NULL;
  image->start_time = NULL;
  image->stop_time = NULL;
  image->fpa_b_temp = 999999;
  image->fpga_temp = 999999;
  image->telescope_temp = 999999;
  image->mode_test = NULL;
  image->pds_identifier = NULL;
  image->mission_phase = NULL;
  image->dqi = 255;
  image->naif_frame_code = 999999;
  image->sample_first_pixel = 255;

  return image;
}

#define MESSAGE_LENGTH 10000

void ensure_shadowcam_image_populated(struct shadowcam_image *image) {
  char messages[MESSAGE_LENGTH] = {0};

  if (image->data_offset == 0)
    strncat(messages, "Data offset not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);

  if (image->tdi_direction == '\0')
    strncat(messages, "TDI direction not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);

  for(int i=0; i < SHC_CHANNELS; i++) {
    if (image->gain[i] == 9999)
      strncat(messages, "Gain value not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
    if (image->offset[i] == 9999)
      strncat(messages, "Offset value not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  }

  for(int i=0; i<6; i++) {
    if (image->bterm[i] == 9999)
      strncat(messages, "BTerm value not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
    if (image->xterm[i] == 9999 && i != 5)
      strncat(messages, "XTerm value not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  }

  if (image->fpa_a_temp == 999999)
    strncat(messages, "FPA A Temperature not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->line_rate_ms == 999999)
    strncat(messages, "Line Rate (ms) not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->lines == 999999)
    strncat(messages, "Lines not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->samples == 9999)
    strncat(messages, "Samples not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->target_name == NULL)
    strncat(messages, "Target Name not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->line_rate_code == 9999)
    strncat(messages, "Line Rate Code not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->preroll == 9999)
    strncat(messages, "Preroll not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->klines == 9999)
    strncat(messages, "KLines not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->preroll_time == NULL)
    strncat(messages, "Preroll Time not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->start_time == NULL)
    strncat(messages, "Start Time not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->stop_time == NULL)
    strncat(messages, "Stop Time not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->fpa_b_temp == 999999)
    strncat(messages, "FPA B Temperature not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->fpga_temp == 999999)
    strncat(messages, "FPGA Temperature not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->telescope_temp == 999999)
    strncat(messages, "Telescope Temperature not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->mode_test == NULL)
    strncat(messages, "Mode test not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->pds_identifier == NULL)
    strncat(messages, "PDS identifier not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->mission_phase == NULL)
    strncat(messages, "Mission phase not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->dqi == 255)
    strncat(messages, "Data quality ID not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->naif_frame_code == 999999)
    strncat(messages, "NAIF frame code not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);
  if (image->sample_first_pixel == 255)
    strncat(messages, "Sample first pixel not initialized.\n", MESSAGE_LENGTH - strlen(messages) - 1);

  if (strnlen(messages, MESSAGE_LENGTH - strlen(messages) - 1) != 0) {
    printf("**** EDR Label Errors Encountered ****\n");
    printf("%s", messages);
    exit(1);
  }

  printf("EDR label is fully populated.\n");
}

void free_shadowcam_image(struct shadowcam_image* image) {
  puts("Freeing ShadowCam image struct.");

  free(image->filename);

  if (image->file != NULL)
    fclose(image->file);

  if (image->data != NULL)
    free(image->data);

  if (image->target_name != NULL)
    free(image->target_name);

  if (image->preroll_time != NULL)
    free(image->preroll_time);

  if (image->start_time != NULL)
    free(image->start_time);

  if (image->stop_time != NULL)
    free(image->stop_time);

  if (image->mode_test != NULL)
    free(image->mode_test);

  if (image->pds_identifier != NULL)
    free(image->pds_identifier);

  if (image->mission_phase != NULL)
    free(image->mission_phase);

  free(image);
}

size_t data_index(uint32_t line, uint8_t channel, uint16_t pixel) {
  return (line * (uint32_t)(SHC_CHANNELS * SHC_AFE_WIDTH) + channel * SHC_AFE_WIDTH + pixel);
}
